import java.applet.Applet;
import java.awt.*;
/**
 * Applet which simulates the Z3 (1941 Konrad Zuse) adder.
 * It realizes a 8-bit integer adder.
 * @author Cneyt Gktekin
 * @version 2.5
 */
public class Adder extends Applet implements Runnable {
  // needed for -double buffering-
  Dimension offDimension;
  Image offImage;
  Graphics offGraphics;
  /** number of Registers (relays). */
  int relays=8;
  /** Control variables. */
  boolean on=false;
  boolean reset=false;
  boolean add=true;
  boolean back1=false;
  boolean back2=false;
  boolean back3=false;
  /** Boolean representation of the register relays. */
  boolean[] relayA;
  boolean[] relayB;
  boolean[] relayC=new boolean[relays];
  boolean[] relayD=new boolean[relays];
  // Image handling
  /** Control images. */
  Image onDI;
  Image onAI;
  Image offI;
  Image resetOn;
  Image resetOff;
  Image backI;
  Image fwdI;
  Image addON;
  Image addOFF;
  Image subON;
  Image subOFF;
  /** Registers. */
  Image relayON;
  Image relayOFF;
  /** Phase I images. */
  Image line1A;
  Image[] xorA;
  Image line1S;
  Image[] xorS;
  /** Phase II images. */
  Image line2A;
  Image[] andA;
  Image line2S;
  Image[] andS;
  Image[] bd;
  /** Phase III images. */
  Image line3;
  Image [] be;
  /** Media tracker to keep loading under control. */
  MediaTracker tracker;
  /** The main animation thread. */
  private Thread thread=null;
  /** 
   * Method realizes the reset button (C).
   * It clears all relay arrays, temp arrays and 
   * back/forward switches for each 
   * phase of the circuit.
   */
  private void c() {
    // Initializing relays (registers) with OFF
    relayA=new boolean[relays];
    relayB=new boolean[relays];
    back1=false;
    back2=false;
    back3=false;
    for(int i=0; i<relays; i++) {
      relayA[i]=false;
      relayB[i]=false;
    }
  }
  /**
   * Loading Images and some other initialisation.
   */
  public void init() {
    // Initializing relays
    c();
    // Loading images
    // Control images
    tracker=new MediaTracker(this);
    onDI=getImage(getCodeBase(),"Images/onD.gif");
    tracker.addImage(onDI,0);
    onAI=getImage(getCodeBase(),"Images/onA.gif");
    tracker.addImage(onAI,0);
    offI=getImage(getCodeBase(),"Images/off.gif");
    tracker.addImage(offI,0);
    resetOn=getImage(getCodeBase(),"Images/resetOn.gif");
    tracker.addImage(resetOn,0);
    resetOff=getImage(getCodeBase(),"Images/resetOff.gif");
    tracker.addImage(resetOff,0);
    backI=getImage(getCodeBase(),"Images/back.gif");
    tracker.addImage(backI,0);
    fwdI=getImage(getCodeBase(),"Images/fwd.gif");
    tracker.addImage(fwdI,0);
    addON=getImage(getCodeBase(),"Images/addON.gif");
    tracker.addImage(addON,0);
    addOFF=getImage(getCodeBase(),"Images/addOFF.gif");
    tracker.addImage(addOFF,0);
    subON=getImage(getCodeBase(),"Images/subON.gif");
    tracker.addImage(subON,0);
    subOFF=getImage(getCodeBase(),"Images/subOFF.gif");
    tracker.addImage(subOFF,0);
    // relay images
    relayON=getImage(getCodeBase(),"Images/relayON.gif");
    tracker.addImage(relayON,1);
    relayOFF=getImage(getCodeBase(),"Images/relayOFF.gif");
    tracker.addImage(relayOFF,2);
    // 1st phase images
    line1A=getImage(getCodeBase(),"Images/Add/line1.gif");
    tracker.addImage(line1A,3);
    line1S=getImage(getCodeBase(),"Images/Sub/line1.gif");
    tracker.addImage(line1S,4);
    xorA=new Image[4];
    for(int i=0; i<4; i++) {
      xorA[i]=getImage(getCodeBase(),"Images/Add/xor"+i+".gif");
      tracker.addImage(xorA[i],5);
    }
    xorS=new Image[4];
    for(int i=0; i<4; i++) {
      xorS[i]=getImage(getCodeBase(),"Images/Sub/xor"+i+".gif");
      tracker.addImage(xorS[i],6);
    }
    // 2nd phase images
    line2A=getImage(getCodeBase(),"Images/Add/line2.gif");
    tracker.addImage(line2A,7);
    line2S=getImage(getCodeBase(),"Images/Sub/line2.gif");
    tracker.addImage(line2S,8);
    andA=new Image[4];
    for(int i=0; i<4; i++) {
      andA[i]=getImage(getCodeBase(),"Images/Add/and"+i+".gif");
      tracker.addImage(andA[i],9);
    }
    andS=new Image[4];
    for(int i=0; i<4; i++) {
      andS[i]=getImage(getCodeBase(),"Images/Sub/and"+i+".gif");
      tracker.addImage(andS[i],10);
    }
    bd=new Image[4];
    for(int i=0; i<4; i++) {
      bd[i]=getImage(getCodeBase(),"Images/bd"+i+".gif");
      tracker.addImage(bd[i],11);
    }
    // 3rd phase images
    line3=getImage(getCodeBase(),"Images/line3.gif");
    tracker.addImage(line3,0);
    be=new Image[8];
    for(int i=0; i<8; i++) {
      be[i]=getImage(getCodeBase(),"Images/be"+i+".gif");
      tracker.addImage(be[i],12);
    }
  }
  public void start() {
    if (thread==null) {
      thread=new Thread(this);
      thread.start();
    }
    else {
      thread=null;
      thread=new Thread(this);
      thread.start();
    }
  }
  public void stop() {
    thread=null;
  }
  /** 
   * Standard mouse event Method.
   * In this Method the clickable areas are defined.
   * ON/OFF, Reset(C), +/(-), back/forward, relay line A,
   * relay line B.
   * @param e MouseEvent
   * @param x x coordinate 
   * @param y y coordinate
   */
  public boolean mouseDown(Event e,int x,int y) {
    // Managing mouse behaviour on OFF button
    if((x>430)&&(x<460)&&(y<40)&&(y>10)) {
      back1=false;
      back2=false;
      back3=false;
      on=false;
      start();
    }
    // Managing mouse behaviour on ON button
    if((x>460)&&(x<490)&&(y<40)&&(y>10)) {
      on=true;start();
    }
    // Managing mouse behaviour on Reset(C) button
    if((x>500)&&(x<530)&&(y<40)&&(y>10)) {
      reset=(reset ? false:true);start();
    }
    // Managing mouse behaviour on add button
    if((x>40)&&(x<70)&&(y<120)&&(y>90)) {
      add=(add ? false:true);start();
    }
    // Managing mouse behaviour on sub button
    if((x>10)&&(x<40)&&(y<120)&&(y>90)) {
      add=(add ? false:true);start();
    }
    // Managing mouse behaviour on back/fwd button in phase 1
    if((x>460)&&(x<500)&&(y<230)&&(y>200)) {
      back1=true;
      back2=true;
      back3=true;
      start();
    }
    if((x>500)&&(x<540)&&(y<230)&&(y>200)) {
      back1=false;
      back2=false;
      back3=false;
      start();
    }
    // Managing mouse behaviour on back/fwd button in phase 2
    if((x>460)&&(x<500)&&(y<330)&&(y>300)) {
      back2=true;
      start();
    }
    if((x>500)&&(x<540)&&(y<330)&&(y>300)) {
      back2=false;
      start();
    }
    // Managing mouse behaviour on back/fwd button in phase 3
    if((x>460)&&(x<500)&&(y<490)&&(y>460)) {
      back3=true;
      start();
    }
    if((x>500)&&(x<540)&&(y<490)&&(y>460)) {
      back3=false;
      start();
    }
    // Managing mouse behaviour of A register relays
    if((x>100)&&(x<115)&&(y<45)&&(y>7)) {
      relayA[0]=(relayA[0] ? false:true);
      start();
    }
    if((x>140)&&(x<155)&&(y<47)&&(y>7)) {
      relayA[1]=(relayA[1] ? false:true);
      start();
    }
    if((x>180)&&(x<195)&&(y<47)&&(y>7)) {
      relayA[2]=(relayA[2] ? false:true);
      start();
    }
    if((x>220)&&(x<235)&&(y<47)&&(y>7)) {
      relayA[3]=(relayA[3] ? false:true);
      start();
    }
    if((x>260)&&(x<275)&&(y<47)&&(y>7)) {
      relayA[4]=(relayA[4] ? false:true);
      start();
    }
    if((x>300)&&(x<315)&&(y<47)&&(y>7)) {
      relayA[5]=(relayA[5] ? false:true);
      start();
    }
    if((x>340)&&(x<355)&&(y<47)&&(y>7)) {
      relayA[6]=(relayA[6] ? false:true);
      start();
    }
    if((x>380)&&(x<395)&&(y<47)&&(y>7)) {
      relayA[7]=(relayA[7] ? false:true);
      start();
    }
   // Managing mouse behaviour of B register relay
   if((x>100)&&(x<115)&&(y<105)&&(y>65)) {
      relayB[0]=(relayB[0] ? false:true);
      start();
    }
    if((x>140)&&(x<155)&&(y<105)&&(y>65)) {
      relayB[1]=(relayB[1] ? false:true);
      start();
    }
    if((x>180)&&(x<195)&&(y<105)&&(y>65)) {
      relayB[2]=(relayB[2] ? false:true);
      start();
    }
    if((x>220)&&(x<235)&&(y<105)&&(y>65)) {
      relayB[3]=(relayB[3] ? false:true);
      start();
    }
    if((x>260)&&(x<275)&&(y<105)&&(y>65)) {
      relayB[4]=(relayB[4] ? false:true);
      start();
    }
    if((x>300)&&(x<315)&&(y<105)&&(y>65)) {
      relayB[5]=(relayB[5] ? false:true);
      start();
    }
    if((x>340)&&(x<355)&&(y<105)&&(y>65)) {
      relayB[6]=(relayB[6] ? false:true);
      start();
    }
    if((x>380)&&(x<395)&&(y<105)&&(y>65)) {
      relayB[7]=(relayB[7] ? false:true);
      start();
    }
    return true;
  }
  /** 
   * Method that puts the relay lines (A/B) and all
   * control buttons on the sheet.
   */  
  private void putRegisterC() {
    int dist=100;
    // Registers
    for(int i=0; i<relays; i++) {
      if(relayA[i]) offGraphics.drawImage(relayON,dist,7,this);
      else offGraphics.drawImage(relayOFF,dist,7,this);
      if(relayB[i]) offGraphics.drawImage(relayON,dist,65,this);
      else offGraphics.drawImage(relayOFF,dist,65,this);
      dist+=40;
    }
    // Placing 1/0 under each register
    dist=109;
    for(int i=0; i<relays; i++) {
      if(relayA[i]) offGraphics.drawString("1",dist,60);
      else offGraphics.drawString("0",dist,60);
      if(relayB[i]) offGraphics.drawString("1",dist,118);
      else offGraphics.drawString("0",dist,118);
      dist+=40;
    }
    // Placing on/off button on sheet
    if(on) {
      offGraphics.drawImage(offI,430,10,this);
      offGraphics.drawImage(onAI,460,10,this);
    }
    else {
      offGraphics.drawImage(offI,430,10,this);
      offGraphics.drawImage(onDI,460,10,this);
    }
    // Placing reset button on sheet
    offGraphics.drawImage(resetOff,500,10,this);
    if(reset) {
      offGraphics.drawImage(resetOff,500,10,this);
      c();
      offGraphics.drawImage(resetOn,500,10,this);
      reset=false;
      on=false;
    }
    // Placing sub/add button on sheet
    if(add) {
      offGraphics.drawImage(addON,40,90,this);
      offGraphics.drawImage(subOFF,10,90,this);
    }
    else {
      offGraphics.drawImage(addOFF,40,90,this);
      offGraphics.drawImage(subON,10,90,this);
    }
  }
  /** 
   * Method that puts phase I, II and the back/forward arrows
   * which belong to each phase.
   */
  private void putAdditionC() {
    // Methods temp variables
    boolean[] temp1A=new boolean[relays];
    boolean[] temp1B=new boolean[relays];
    boolean[] temp2A=new boolean[relays];
    boolean[] temp2B=new boolean[relays];
    boolean[] temp2C=new boolean[relays];
    boolean[] temp2D=new boolean[relays];
    boolean[] temp3A=new boolean[relays];
    boolean[] temp3C=new boolean[relays];
    boolean[] temp3D=new boolean[relays];
    // Draw, but don't change switches until button ON/OFF is ON
    if(on) {
      for(int i=0; i<relays;i++) {
        temp1A[i]=relayA[i];
        temp1B[i]=relayB[i];
        temp2A[i]=relayA[i];
        temp2B[i]=relayB[i];
        temp2C[i]=relayC[i];
        temp3A[i]=relayA[i];
        temp3C[i]=relayC[i];
        temp3D[i]=relayD[i];
      }
    }
    // Phase 1
    // Placing back/fwd button on sheet
    offGraphics.drawImage(backI,460,200,this);
    offGraphics.drawImage(fwdI,500,200,this);
    if(on) {
      if(back1) {
        for(int i=0; i<relays; i++) {
          temp1A[i]=false;
          temp1B[i]=false;
        }
      }
      else {
        for(int i=0; i<relays; i++) {
          temp1A[i]=relayA[i];
          temp1B[i]=relayB[i];
        }
      }
    }
    // Placing the main line
    offGraphics.drawImage(line1A,5,120,this);
    // Placing xors on sheet
    int dist=102;
    int dist2=107;
    for(int i=0; i<relays; i++) {
      if(temp1A[i]&&temp1B[i]) {
        relayC[i]=false;
        offGraphics.drawImage(xorA[2],dist,133,this);
        offGraphics.drawString("0",dist2,255);
      }
      if(temp1A[i]&&!temp1B[i]) {
        relayC[i]=true;
        offGraphics.drawImage(xorA[3],dist,133,this);
        offGraphics.drawString("1",dist2,255);
      }
      if(!temp1A[i]&&temp1B[i]) {
        relayC[i]=true;
        offGraphics.drawImage(xorA[1],dist,133,this);
        offGraphics.drawString("1",dist2,255);
      }
      if(!temp1A[i]&&!temp1B[i]) {
        relayC[i]=false;
        offGraphics.drawImage(xorA[0],dist,133,this);
        offGraphics.drawString("0",dist2,255);
      }
      dist+=40;
      dist2+=40;
    }
    // Phase 2
    // Placing the main line
    offGraphics.drawImage(line2A,5,270,this);
    // Placing back/fwd button on sheet
    offGraphics.drawImage(backI,460,300,this);
    offGraphics.drawImage(fwdI,500,300,this);
    if(on) {
      if(back2) {
        for(int i=0; i<relays; i++) {
          temp2A[i]=false;
          temp2B[i]=false;
          temp2C[i]=false;
          temp2D[i]=false;
        }
      }
      else {
        for(int i=0; i<relays; i++) {
          temp2A[i]=relayA[i];
          temp2B[i]=relayB[i];
          temp2C[i]=relayC[i];
          temp2D[i]=relayD[i];
        }
      }
    }
    // Placing the and switches
    dist=100;
    for(int i=1; i<relays; i++) {
      if(!temp2A[i]&&!temp2B[i]) offGraphics.drawImage(andA[0],dist,268,this);
      if(temp2A[i]&&!temp2B[i]) offGraphics.drawImage(andA[1],dist,268,this);
      if(!temp2A[i]&&temp2B[i]) offGraphics.drawImage(andA[2],dist,268,this);
      if(temp2A[i]&&temp2B[i]) offGraphics.drawImage(andA[3],dist,268,this);
      dist+=47;
    }
    // Placing the bd registers
    dist=348;
    dist2=370;
    // most right bd register is always 0 (false)
    relayD[7]=false;
    for(int i=relays-2; i>=0; i--) {
      if(!temp2A[i+1]&&!temp2B[i+1]) {
          relayD[i]=false;
          offGraphics.drawImage(bd[0],dist,342,this);
      }
      else if(!temp2A[i+1]&&temp2B[i+1]) {
        if(relayD[i+1]) {
          relayD[i]=true;
          offGraphics.drawImage(bd[3],dist,342,this);
        }
        else {
          relayD[i]=false;
          offGraphics.drawImage(bd[1],dist,342,this);
        }
      }
      else if(temp2A[i+1]&&!temp2B[i+1]) {
        if(relayD[i+1]) {
          relayD[i]=true;
          offGraphics.drawImage(bd[3],dist,342,this);
        }
        else {
          relayD[i]=false;
          offGraphics.drawImage(bd[1],dist,342,this);
        }
      }
      else /*(temp2A[i+1]&&temp2B[i+1])*/ {
        relayD[i]=true;
        offGraphics.drawImage(bd[2],dist,342,this);
      }
      dist-=47;
      dist2-=50;
    }
    // Placing 1/0 under each bd register
    dist=370;
    for(int i=relays-2; i>=0; i--) {
      if(relayD[i]) offGraphics.drawString("1",dist,402);
      else offGraphics.drawString("0",dist,402);
      dist-=47;
    }
    dist=420;
    offGraphics.drawString("0",dist,402);
    // Phase 3
    // Placing back/fwd button on sheet
    offGraphics.drawImage(backI,460,460,this);
    offGraphics.drawImage(fwdI,500,460,this);
    offGraphics.drawImage(line3,5,410,this);
    if(on) {
      if(back3) {
        for(int i=0; i<relays; i++) {
          temp3A[i]=false;
          temp3C[i]=false;
          temp3D[i]=false;
        }
      }
      else {
        for(int i=0; i<relays; i++) {
          temp3A[i]=relayA[i];
          temp3C[i]=relayC[i];
          temp3D[i]=relayD[i];
        }
      }
    }
    dist=45;
    dist2=54;
    for(int i=0; i<relays; i++) {
      if(temp3A[i]&&temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[7],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      if(temp3A[i]&&!temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[6],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(!temp3A[i]&&temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[5],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      if(temp3A[i]&&temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[4],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(temp3A[i]&&!temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[3],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      if(!temp3A[i]&&temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[2],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(!temp3A[i]&&!temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[1],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(!temp3A[i]&&!temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[0],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      dist+=50;
      dist2+=50;
    }
  }
  /** 
   * Doing the same as putAddC() only using the ...S images.
   * @see Adder#putAdditionC
   */
  private void putSubtractionC() {
    // Methods temp variables
    boolean[] temp1A=new boolean[relays];
    boolean[] temp1B=new boolean[relays];
    boolean[] temp2A=new boolean[relays];
    boolean[] temp2B=new boolean[relays];
    boolean[] temp2C=new boolean[relays];
    boolean[] temp2D=new boolean[relays];
    boolean[] temp3A=new boolean[relays];
    boolean[] temp3C=new boolean[relays];
    boolean[] temp3D=new boolean[relays];
    // Draw, but don't change switches until button ON/OFF is ON
    if(on) {
      for(int i=0; i<relays;i++) {
        temp1A[i]=relayA[i];
        temp1B[i]=relayB[i];
        temp2A[i]=relayA[i];
        temp2B[i]=relayB[i];
        temp2C[i]=relayC[i];
        temp3A[i]=relayA[i];
        temp3C[i]=relayC[i];
        temp3D[i]=relayD[i];
      }
    }
    else {
      for(int i=0; i<relays;i++) {
        temp2C[i]=true;
        temp3C[i]=true;
        temp3D[i]=true;
      }
    }
    // Phase 1
    // Placing back/fwd button on sheet
    offGraphics.drawImage(backI,460,200,this);
    offGraphics.drawImage(fwdI,500,200,this);
    if(on) {
      if(back1) {
        for(int i=0; i<relays; i++) {
          temp1A[i]=false;
          temp1B[i]=false;
        }
      }
      else {
        for(int i=0; i<relays; i++) {
          temp1A[i]=relayA[i];
          temp1B[i]=relayB[i];
        }
      }
    }
    offGraphics.drawImage(line1S,5,120,this);
    int dist=102;
    int dist2=107;
    for(int i=0; i<relays; i++) {
      if(temp1A[i]&&temp1B[i]) {
        relayC[i]=true;
        offGraphics.drawImage(xorS[2],dist,133,this);
        offGraphics.drawString("1",dist2,255);
      }
      if(temp1A[i]&&!temp1B[i]) {
        relayC[i]=false;
        offGraphics.drawImage(xorS[3],dist,133,this);
        offGraphics.drawString("0",dist2,255);
      }
      if(!temp1A[i]&&temp1B[i]) {
        relayC[i]=false;
        offGraphics.drawImage(xorS[1],dist,133,this);
        offGraphics.drawString("0",dist2,255);
      }
      if(!temp1A[i]&&!temp1B[i]) {
        relayC[i]=true;
        offGraphics.drawImage(xorS[0],dist,133,this);
        offGraphics.drawString("1",dist2,255);
      }
      dist+=40;
      dist2+=40;
    }
    // Phase 2
    // Placing the main line
    offGraphics.drawImage(line2S,5,270,this);
    // Placing back/fwd button on sheet
    offGraphics.drawImage(backI,460,300,this);
    offGraphics.drawImage(fwdI,500,300,this);
    if(on) {
      if(back2) {
        for(int i=0; i<relays; i++) {
          temp2A[i]=false;
          temp2B[i]=false;
          temp2C[i]=true;
          temp2D[i]=true;
        }
      }
      else {
        for(int i=0; i<relays; i++) {
          temp2A[i]=relayA[i];
          temp2B[i]=relayB[i];
          temp2C[i]=relayC[i];
          temp2D[i]=relayD[i];
        }
      }
    }
    // Placing the AND switches
    dist=100;
    for(int i=1; i<relays; i++) {
      if(!temp2A[i]&&!temp2B[i]) offGraphics.drawImage(andS[0],dist,268,this);
      if(temp2A[i]&&!temp2B[i]) offGraphics.drawImage(andS[1],dist,268,this);
      if(!temp2A[i]&&temp2B[i]) offGraphics.drawImage(andS[2],dist,268,this);
      if(temp2A[i]&&temp2B[i]) offGraphics.drawImage(andS[3],dist,268,this);
      dist+=47;
    }
    // Placing the Bd registers
    dist=348;
    dist2=370;
    // the most right relay is always 1
    relayD[7]=true;
    for(int i=relays-2; i>=0; i--) {
      if(!temp2A[i+1]&&!temp2B[i+1]) {
        if(relayD[i+1]) {
          relayD[i]=true;
          offGraphics.drawImage(bd[3],dist,342,this);
        }
        else {
          relayD[i]=false;
          offGraphics.drawImage(bd[1],dist,342,this);
        }
      }
      else if(temp2A[i+1]&&!temp2B[i+1]) {
        relayD[i]=true;
        offGraphics.drawImage(bd[2],dist,342,this);
      }
      else if(!temp2A[i+1]&&temp2B[i+1]) {
        relayD[i]=false;
        offGraphics.drawImage(bd[0],dist,342,this);
      }
      else /*(temp2A[i+1]&&temp2B[i+1])*/ {
        if(relayD[i+1]) {
          relayD[i]=true;
          offGraphics.drawImage(bd[3],dist,342,this);
        }
        else {
          relayD[i]=false;
          offGraphics.drawImage(bd[1],dist,342,this);
        }
      }
      dist-=47;
      dist2-=50;
    }
    // Placing 1/0 under each bd register
    dist=370;
    for(int i=relays-2; i>=0; i--) {
      if(relayD[i]) offGraphics.drawString("1",dist,402);
      else offGraphics.drawString("0",dist,402);
      dist-=47;
    }
    dist=420;
    offGraphics.drawString("1",dist,402);
    // Phase 3
    // Placing back/fwd button on sheet
    offGraphics.drawImage(backI,460,460,this);
    offGraphics.drawImage(fwdI,500,460,this);
    offGraphics.drawImage(line3,5,410,this);
    if(on) {
      if(back3) {
        for(int i=0; i<relays; i++) {
          temp3A[i]=false;
          temp3C[i]=true;
          temp3D[i]=true;
        }
      }
      else {
        for(int i=0; i<relays; i++) {
          temp3A[i]=relayA[i];
          temp3C[i]=relayC[i];
          temp3D[i]=relayD[i];
        }
      }
    }
    dist=45;
    dist2=54;
    for(int i=0; i<relays; i++) {
      if(temp3A[i]&&temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[7],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      if(temp3A[i]&&!temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[6],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(!temp3A[i]&&temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[5],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      if(temp3A[i]&&temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[4],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(temp3A[i]&&!temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[3],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      if(!temp3A[i]&&temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[2],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(!temp3A[i]&&!temp3C[i]&&temp3D[i]) {
        offGraphics.drawImage(be[1],dist,411,this);
        offGraphics.drawString("1",dist2,530);
      }
      if(!temp3A[i]&&!temp3C[i]&&!temp3D[i]) {
        offGraphics.drawImage(be[0],dist,411,this);
        offGraphics.drawString("0",dist2,530);
      }
      dist+=50;
      dist2+=50;
    }
  }
  /**
   * This method realizes an increasing progross bar.
   * @param g graphics variable in which the bar will be placed.
   * @param images number of images.
   */
  public void buildProgress(Graphics g,int images) {
    int factor=30;
    // scaling
    Dimension d=size();
    // Setting color
    g.setColor(Color.blue);
    int x1=d.width/2-(images*factor)/2;
    // drawing a rectangle round the bar
    g.drawRect(x1,d.height/2+20,factor*images,20);
    // realizing the increasing bar
    for(int i=0; i<=images; i++) {
      g.fillRect(x1,d.height/2+20,i*factor,20);
      try {
        tracker.waitForID(i);
      } catch (InterruptedException ie) {}
    }
    g.setColor(Color.black);
  }
  public void run() {
    try {
      // Start downloading the images. Wait until they're loaded
      // before requesting repaints.
      tracker.waitForAll();
    } catch (InterruptedException ie) {}
    while(Thread.currentThread()==thread) {
      // Working with reduced performance
      Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
      repaint();
      try {
        thread.sleep(500);
      }catch (InterruptedException ie) {}
    }
  }
  public void paint(Graphics g) {
    update(g);
  }
  public void update(Graphics g) {
    // scaling
    Dimension d=size();
    // wait until all images are loaded
    if (!tracker.checkAll()) {
      g.setColor(Color.white);
      g.fillRect(0, 0, d.width, d.height);
      g.setColor(Color.black);
      g.drawString("Please wait, loading...",(d.width/2-80),d.height/2);
      buildProgress(g,12);
    }
    // now start drawing !!!
    else {
		  if ( (offGraphics==null) 
		  		|| (d.width!=offDimension.width) 
			  	|| (d.height!=offDimension.height) ) {
        offDimension=d;
        offImage=createImage(d.width, d.height);
        offGraphics=offImage.getGraphics();
      }
      // drawing white sheet
      offGraphics.setColor(Color.white);
      offGraphics.fillRect(0,0,d.width,d.height);
      offGraphics.setColor(Color.black);
      // drawing denotation
      Font font=offGraphics.getFont();
      
      offGraphics.setFont(new Font("Standard",Font.BOLD,24));
      offGraphics.setColor(Color.lightGray);
      offGraphics.drawString("A",75,40);
      offGraphics.drawString("B",75,100);
      offGraphics.drawString("S",70,235);
      offGraphics.drawString("C",50,385);
      offGraphics.drawString("R",30,510);
      offGraphics.setColor(Color.gray);
      offGraphics.drawString("1",10,190);
      offGraphics.drawString("2",10,320);
      offGraphics.drawString("3",10,450);
      offGraphics.setFont(font);
      // drawing Registers
      putRegisterC();
      // draw phase I and II
      if(add) putAdditionC();
      else putSubtractionC();
      // painting Buffer on Applet
      g.drawImage(offImage,0,0,this);
    }
  }
}
