import java.applet.Applet;
import java.awt.*;
/**
 * Applet which simulates the Z3 (1941 Konrad Zuse) Normalizer.
 * It simulates a 16-bit normalizer.
 * @author Cneyt Gktekin
 * @version 1.5
 */
public class Normalizer extends Applet implements Runnable {
  // needed for -double buffering-
  Dimension offDimension;
  Image offImage;
  Graphics offGraphics;
  /** number of bits. */
  int nr=17;
  /** Some booleans to control the circuit. */
  /** ON/OFF button. */
  boolean on=false;
  /** Reset button. */
  boolean c=false;
  /** Temp indicates if the line is on power. */
  boolean lineActive=false;
  /** Bits which are set by the bit buttons. */
  boolean[] bits;
  /** Array that helps to draw the right color of the be switches. */
  boolean[] power;
  /** Array indicates which be switch is activated thru the line. */
  boolean[] thruLinePower;
  // Image handling
  /** Images for the control buttons.*/
  Image onAI;
  Image onDI;
  Image offI;
  Image resetONI;
  Image resetOFFI;
  /** Image for on bit and off bit. */
  Image bit0I;
  Image bit1I;
  /** Images needed for all levels. */
  Image[] beI;
  /** first level images. */
  Image[] lineI;
  Image[] frontI;
  Image[] rearI;
  /** second level images. */
  Image[] front2I;
  Image[] be2I;
  Image[] line2I;
  /** third level images. */
  Image[] front3I;
  Image[] frame3I;
  Image[] be32I;
  Image[] line3I;
  /** fourth level images. */
  Image[] front4I;
  Image[] frame4I;
  /** Media tracker to keep loading under control. */
  MediaTracker tracker;
  /** The main simulation thread. */
  private Thread thread=null;
  /** 
   * Method realizes the reset button (C).
   * It creates a new array with false values.
   */
  private void c() {
    bits=new boolean[nr];
    power=new boolean[nr];
    thruLinePower=new boolean[nr];
  }
  /**
   * Loading Images and some other initialization.
   */
  public void init() {
    // Initializing bits
    c();
    // Loading images
    // first create a new MediaTracker object
    tracker=new MediaTracker(this);
    // control images
    onAI=getImage(getCodeBase(),"Images/onA.gif");
    tracker.addImage(onAI,0);
    onDI=getImage(getCodeBase(),"Images/onD.gif");
    tracker.addImage(onDI,0);
    offI=getImage(getCodeBase(),"Images/off.gif");
    tracker.addImage(offI,0);
    resetOFFI=getImage(getCodeBase(),"Images/resetOff.gif");
    tracker.addImage(resetOFFI,0);
    resetONI=getImage(getCodeBase(),"Images/resetOn.gif");
    tracker.addImage(resetONI,0);
    bit1I=getImage(getCodeBase(),"Images/bit1.gif");
    tracker.addImage(bit1I,0);
    bit0I=getImage(getCodeBase(),"Images/bit0.gif");
    tracker.addImage(bit0I,0);
    // Images needed for all levels
    // be switches
    beI=new Image[4];
    for(int i=0; i<4; i++) {
      beI[i]=getImage(getCodeBase(),"Images/be"+i+".gif");
      tracker.addImage(beI[i],1);
    }
    // first level images
    // front switch
    frontI=new Image[2];
    for(int i=0; i<2; i++) {
      frontI[i]=getImage(getCodeBase(),"Images/Fm/front"+i+".gif");
      tracker.addImage(frontI[i],2);
    }
    // rear switch
    rearI=new Image[2];
    for(int i=0; i<2; i++) {
      rearI[i]=getImage(getCodeBase(),"Images/Fm/rear"+i+".gif");
      tracker.addImage(rearI[i],3);
    }
    // line
    lineI=new Image[2];
    for(int i=0; i<2; i++) {
      lineI[i]=getImage(getCodeBase(),"Images/Fm/line"+i+".gif");
      tracker.addImage(lineI[i],4);
    }
    // second level images
    // be switches (between line and be)
    be2I=new Image[4];
    for(int i=0; i<4; i++) {
      be2I[i]=getImage(getCodeBase(),"Images/Fl/be"+i+".gif");
      tracker.addImage(be2I[i],5);
    }
    // front switch
    front2I=new Image[4];
    for(int i=0; i<4; i++) {
      front2I[i]=getImage(getCodeBase(),"Images/Fl/front"+i+".gif");
      tracker.addImage(front2I[i],6);
    }
    // line
    line2I=new Image[2];
    for(int i=0; i<2; i++) {
      line2I[i]=getImage(getCodeBase(),"Images/Fl/line"+i+".gif");
      tracker.addImage(line2I[i],7);
    }
    // third level images
    // front switch
    front3I=new Image[4];
    for(int i=0; i<4; i++) {
      front3I[i]=getImage(getCodeBase(),"Images/Fk/front"+i+".gif");
      tracker.addImage(front3I[i],8);
    }
    // frame unit
    frame3I=new Image[4];
    for(int i=0; i<4; i++) {
      frame3I[i]=getImage(getCodeBase(),"Images/Fk/frame"+i+".gif");
      tracker.addImage(frame3I[i],9);
    }
    // be switches in the frame unit
    be32I=new Image[4];
    for(int i=0; i<4; i++) {
      be32I[i]=getImage(getCodeBase(),"Images/Fk/be"+i+".gif");
      tracker.addImage(be32I[i],10);
    }
    // line
    line3I=new Image[2];
    for(int i=0; i<2; i++) {
      line3I[i]=getImage(getCodeBase(),"Images/Fk/line"+i+".gif");
      tracker.addImage(line3I[i],11);
    }
    // fourth level images
    // front piece
    front4I=new Image[4];
    for(int i=0; i<4; i++) {
      front4I[i]=getImage(getCodeBase(),"Images/Fi/front"+i+".gif");
      tracker.addImage(front4I[i],12);
    }
    // frame unit
    frame4I=new Image[4];
    for(int i=0; i<4; i++) {
      frame4I[i]=getImage(getCodeBase(),"Images/Fi/frame"+i+".gif");
      tracker.addImage(frame4I[i],13);
    }
  }
  public void start() {
    if (thread==null) {
      thread=new Thread(this);
      thread.start();
    }
    else {
      thread=null;
      thread=new Thread(this);
      thread.start();
    }
  }
  public void stop() {
    thread=null;
  }
  /** 
   * Standard mouse event Method.
   * In this Method the clickable areas are defined.
   * @param e MouseEvent
   * @param x x coordinate 
   * @param y y coordinate
   */
  public boolean mouseDown(Event e,int x,int y) {
    // Managing behaviour on ON button
    int x1Local=700;
    int y1Local=10;
    if((x>x1Local)&&(x<(x1Local+30))&&(y<y1Local+30)&&(y>y1Local)) {
      on=true;start();
    }
    // Managing behaviour on OFF button
    x1Local=670;
    y1Local=10;
    if((x>x1Local)&&(x<(x1Local+30))&&(y<y1Local+30)&&(y>y1Local)) {
      c();start();on=false;start();
    }
    // Managing behaviour on reset button
    x1Local=750;
    y1Local=10;
    if((x>x1Local)&&(x<(x1Local+40))&&(y<y1Local+40)&&(y>y1Local)) {
      c=(c ? false:true);start();
    }
    // Managing behaviour on bit buttons
    x1Local=45;
    y1Local=10;
    int sum=35;
    for(int i=0; i<nr; i++) {
      if((x>x1Local)&&(x<(x1Local+18))&&(y<y1Local+18)&&(y>y1Local)) {
        bits[i]=(bits[i] ? false:true);start();
      }
      x1Local+=sum;
    }
    start();
    return true;
  }
  /** Method that puts the control buttons on the sheet. */
  private void putControlsC() {
    // putting ON/OFF button on sheet
    int distX=670;
    int distY=10;
    if(on) {
      offGraphics.drawImage(offI,distX,distY,this);
      offGraphics.drawImage(onAI,distX+30,distY,this);
    }
    else {
      offGraphics.drawImage(offI,distX,distY,this);
      offGraphics.drawImage(onDI,distX+30,distY,this);
    }
    // putting reset button on sheet
    distX=750;
    distY=10;
    offGraphics.drawImage(resetOFFI,distX,distY,this);
    if(c) {
      offGraphics.drawImage(resetOFFI,distX,distY,this);
      c();start();
      offGraphics.drawImage(resetONI,distX,distY,this);
      c=false;start();
      on=false;start();
    }
  }  
  /** Method that puts the bit buttons on the sheet. */
  private void putBitButtonsC() {
    int distX=45;
    int distY=10;
    int sum=35;
    for(int i=0; i<nr; i++) {
      if(!bits[i]) offGraphics.drawImage(bit0I,distX,distY,this);
      else offGraphics.drawImage(bit1I,distX,distY,this);
      distX+=sum;
    }
  }
  /** Method that puts the first level switches on the sheet. */
  private void putFmC() {
    // Front switch has always power.
    power[0]=true;
    // Putting front switch be(0) on sheet.
    int distX=13;
    int distY=33;
    if(on) {
      if(!bits[0]) offGraphics.drawImage(frontI[0],distX,distY,this);
      else offGraphics.drawImage(frontI[1],distX,distY,this);
    }
    else offGraphics.drawImage(frontI[0],distX,distY,this);
    // Indicating the color of be switches.
    for(int i=1; i<nr; i++) {
      if(!bits[i-1]&&power[i-1]) power[i]=true;
      else power[i]=false;
    }
    // Temp indicates if the line is on power.
    lineActive=false;
    for(int i=1; i<=13; i+=2) {
      if(bits[i]&&power[i]) lineActive=true;
    }
    // Putting line on sheet.
    distX=82;
    distY=40;
    if(on) {
      if(lineActive) offGraphics.drawImage(lineI[1],distX,distY,this);
      else offGraphics.drawImage(lineI[0],distX,distY,this);
    }
    else offGraphics.drawImage(lineI[0],distX,distY,this);
    /*
     * Storing in array thruLinePower which be switch gets power 
     * through the "Fm" line.
     */
    // first restore bits[14] from Fl stage
    thruLinePower[14]=false;
    // bits[13]
    if(bits[13]&&lineActive) {
      thruLinePower[13]=true;
      for(int i=12; i>=2; i--) {
        if(!bits[i]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      thruLinePower[13]=false;
      for(int i=12; i>=2; i--) {
        if(!bits[i]) thruLinePower[i]=false;
        else thruLinePower[i]=false;
      }
    }
    // bits[11]
    if(bits[11]&&lineActive) {
      thruLinePower[11]=true;
      for(int i=10; i>=2; i--) {
        if(!bits[i]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      if(!thruLinePower[12]) {
        thruLinePower[11]=false;
        for(int i=10; i>=2; i--) {
          if(!bits[i]) thruLinePower[i]=false;
          else thruLinePower[i]=false;
        }
      }
    }
    // bits[9]
    if(bits[9]&&lineActive) {
      thruLinePower[9]=true;
      for(int i=8; i>=2; i--) {
        if(!bits[i]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      if(!thruLinePower[10]) {
        thruLinePower[9]=false;
        for(int i=8; i>=2; i--) {
          if(!bits[i]) thruLinePower[i]=false;
          else thruLinePower[i]=false;
        }
      }
    }
    // bits[7]
    if(bits[7]&&lineActive) {
      thruLinePower[7]=true;
      for(int i=6; i>=2; i--) {
        if(!bits[i]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      if(!thruLinePower[8]) {
        thruLinePower[7]=false;
        for(int i=6; i>=2; i--) {
          if(!bits[i]) thruLinePower[i]=false;
          else thruLinePower[i]=false;
        }
      }
    }
    // bits[5]
    if(bits[5]&&lineActive) {
      thruLinePower[5]=true;
      for(int i=4; i>=2; i--) {
        if(!bits[i]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      if (!thruLinePower[6]) {
        thruLinePower[5]=false;
        for(int i=4; i>=2; i--) {        
          if(!bits[i]) thruLinePower[i]=false;
          else thruLinePower[i]=false;
        }
      }
    }
    // bits[3]
    if(bits[3]&&lineActive) {
      thruLinePower[3]=true;
      if(!bits[2]) thruLinePower[2]=true;
      else thruLinePower[2]=false;
    }
    else {
      if(!thruLinePower[4]) {
        thruLinePower[3]=false;
        if(!bits[2]) thruLinePower[2]=false;
        else thruLinePower[2]=false;
      }
    }
    // Putting be(-1) - be(17) switches on sheet.
    distX=47;
    distY=30;
    int sum=35;
    for(int i=1; i<nr; i++) {
      if(on) {
        if(bits[i]) {
          if(power[i]||thruLinePower[i]) offGraphics.drawImage(beI[3],distX,distY,this);
          else offGraphics.drawImage(beI[1],distX,distY,this);
        }
        else {
          if(power[i]||thruLinePower[i]) offGraphics.drawImage(beI[2],distX,distY,this);
          else offGraphics.drawImage(beI[0],distX,distY,this);
        }
      }
      else offGraphics.drawImage(beI[2],distX,distY,this);
      distX+=sum;
    }
    // Putting rear switch b(82) on sheet.
    if(on) {
      if(power[16]) offGraphics.drawImage(rearI[1],distX,distY,this);
      else offGraphics.drawImage(rearI[0],distX,distY,this);
    }
    else offGraphics.drawImage(rearI[1],distX,distY,this);
  }
  /** Method that puts the second level switches on the sheet. */
  public void putFlC() {
    // Putting front switch be(0) on sheet.
    int distX=55;
    int distY=33;
    if(on) {
      if(!bits[0])
        if(bits[1]) offGraphics.drawImage(front2I[3],distX,distY,this);
        else {
          offGraphics.drawImage(front2I[2],distX,distY,this);
          power[0]=true;
        }
      else 
        if(bits[1]) offGraphics.drawImage(front2I[1],distX,distY,this);
        else offGraphics.drawImage(front2I[0],distX,distY,this);
    }
    else offGraphics.drawImage(front2I[2],distX,distY,this);
    // Temp indicates if the line is on power.
    lineActive=false;
    for(int i=2; i<=14; i+=4) {
      if((bits[i]&&power[i])||(bits[i+1]&&power[i+1])) lineActive=true;
    }
    // putting line on sheet
    distX=117;
    distY=77;
    if(on) {
      if(lineActive) offGraphics.drawImage(line2I[1],distX,distY,this);
      else offGraphics.drawImage(line2I[0],distX,distY,this);
    }
    else offGraphics.drawImage(line2I[0],distX,distY,this);

    /*
     * Storing in array thruLinePower which be switch gets power 
     * through the "Fl" line.
     */
    // bits[14] & bits[15]
    if((bits[14]&&lineActive)||(bits[15]&&lineActive)) {
      thruLinePower[14]=true;
      for(int i=13; i>=3; i--) {
        if(!bits[i]&&thruLinePower[i+1]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      thruLinePower[14]=false;
      for(int i=13; i>=3; i--) {
        if(!bits[i]) thruLinePower[i]=false;
        else thruLinePower[i]=false;
      }
    }
    // bits[10] & bits[11]
    if((bits[10]&&lineActive)||(bits[11]&&lineActive)) {
      thruLinePower[10]=true;
      for(int i=9; i>=3; i--) {
        if(!bits[i]&&thruLinePower[i+1]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      if(!thruLinePower[11]) {
        thruLinePower[10]=false;
        for(int i=9; i>=3; i--) {
          if(!bits[i]) thruLinePower[i]=false;
          else thruLinePower[i]=false;
        }
      }
    }
    if(thruLinePower[10]&&!bits[10]) thruLinePower[11]=true;
    // bits[6] & bits[7]
    if((bits[6]&&lineActive)||(bits[7]&&lineActive)) {
      thruLinePower[6]=true;
      for(int i=5; i>=3; i--) {
        if(!bits[i]&&thruLinePower[i+1]) thruLinePower[i]=true;
        else thruLinePower[i]=false;
      }
    }
    else {
      if(!thruLinePower[7]) {
        thruLinePower[6]=false;
        for(int i=5; i>=3; i--) {
          if(!bits[i]) thruLinePower[i]=false;
          else thruLinePower[i]=false;
        }
      }
    }
    if(thruLinePower[6]&&!bits[6]) thruLinePower[7]=true;
    // bits[2] & bits[3]
    if(bits[2]&&lineActive) {
      thruLinePower[2]=true;
      if(!bits[1]) thruLinePower[1]=true;
      else thruLinePower[1]=false;
    }
    else {
      if(!thruLinePower[3]) {
        thruLinePower[2]=false;
        if(!bits[1]) thruLinePower[1]=false;
        else thruLinePower[1]=false;
      }
    }
    // putting be switches on sheet
    distX=82;
    distY=68;
    int sum=35;
    for(int i=2; i<nr-2; i++) {
      if(on) {
        if(bits[i]) {
          if(power[i]||thruLinePower[i]) offGraphics.drawImage(beI[3],distX,distY,this);
          else offGraphics.drawImage(beI[1],distX,distY,this);
        }
        else {
          if(power[i]||thruLinePower[i]) offGraphics.drawImage(beI[2],distX,distY,this);
          else offGraphics.drawImage(beI[0],distX,distY,this);
        }
      }
      else offGraphics.drawImage(beI[2],distX,distY,this);
      distX+=sum;
    }
    // putting be2 switches on sheet
    distX=105;
    distY=76;
    sum=140;
    for(int i=2; i<=14; i+=4) {
      if(on) {
        if(power[i]||thruLinePower[i]) {
          if(bits[i+1]) offGraphics.drawImage(be2I[3],distX,distY,this);
          else offGraphics.drawImage(be2I[2],distX,distY,this);
        }
        else {
          if(bits[i+1]) offGraphics.drawImage(be2I[1],distX,distY,this);
          else offGraphics.drawImage(be2I[0],distX,distY,this);
        }
      }
      else offGraphics.drawImage(be2I[2],distX,distY,this);
      distX+=sum;
    }
  }
  /** Method that puts the third level switches on the sheet. */
  public void putFkC() {
    // indicates if be5 or be6 or be7 is true!
    boolean frameActive=false;
    // Putting front switch be(0) on sheet.
    int distX=89;
    int distY=71;
    if(on) {
      if(!bits[0]&&!bits[1])
        if(bits[2]) offGraphics.drawImage(front3I[3],distX,distY,this);
        else {
          offGraphics.drawImage(front3I[2],distX,distY,this);
        }
      else 
        if(bits[2]) offGraphics.drawImage(front3I[1],distX,distY,this);
        else offGraphics.drawImage(front3I[0],distX,distY,this);
    }
    else offGraphics.drawImage(front3I[2],distX,distY,this);
    // Temp indicates if the line is on power.
    lineActive=false;
    if((bits[4]&&power[4])||(bits[12]&&power[12])) 
      lineActive=true;
    if(power[4]&&(bits[5]||bits[6]||bits[7]))
      lineActive=true;
    if(on) {
        
      if(bits[12]&&lineActive) {
        power[12]=true;
        for(int i=12; i>=5; i--) {
          if(!bits[i-1]&&power[i]) power[i-1]=true;
          frameActive=true;
        }
      }
      if(lineActive&&(bits[13]||bits[14]||bits[15])) {
        power[12]=true;
        for(int i=12; i>=5; i--) {
          if(!bits[i-1]&&power[i]) power[i-1]=true;
          frameActive=true;
        }
      }

      
      // putting be switches in second frame
      if(power[12]) {
        // be13
        distX=487;
        distY=131;
        if(bits[13]) {offGraphics.drawImage(be32I[3],distX,distY,this);frameActive=true;}
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be14
        distX=521;
        distY=143;
        if(bits[14]) {offGraphics.drawImage(be32I[3],distX,distY,this);frameActive=true;}
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be15
        distX=556;
        distY=156;
        if(bits[15]) {offGraphics.drawImage(be32I[3],distX,distY,this);frameActive=true;}
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // putting second frame (be13-be15) on sheet
        distX=455;
        distY=126;
        if(frameActive) {
          offGraphics.drawImage(frame3I[3],distX,distY,this);
          lineActive=true;
        }
        else if(bits[12]&&power[12]) 
          offGraphics.drawImage(frame3I[3],distX,distY,this);
        else if(lineActive&&!frameActive) {
          offGraphics.drawImage(frame3I[2],distX,distY,this);
        }
        else if(lineActive&&frameActive) {
          offGraphics.drawImage(frame3I[3],distX,distY,this);
        }
        else offGraphics.drawImage(frame3I[1],distX,distY,this);
      }
      else {
        if(bits[12]&&lineActive) {
          for(int i=12; i>=5; i--) {if(!bits[i]) power[i]=true;}
        }
        // be13
        distX=487;
        distY=131;
        if((lineActive&&bits[13])||(lineActive&&bits[14])||(lineActive&&bits[15]))
          frameActive=true;
        if(lineActive&&bits[13]&&frameActive) 
          offGraphics.drawImage(be32I[3],distX,distY,this);
        else if(!lineActive&&bits[13]) 
          offGraphics.drawImage(be32I[1],distX,distY,this);
        else if(lineActive&&!bits[13]&&frameActive) 
          offGraphics.drawImage(be32I[2],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be14
        distX=521;
        distY=143;
        if(lineActive&&bits[14]&&frameActive) 
          offGraphics.drawImage(be32I[3],distX,distY,this);
        else if(!lineActive&&bits[14]) 
          offGraphics.drawImage(be32I[1],distX,distY,this);
        else if(lineActive&&!bits[14]&&frameActive) 
          offGraphics.drawImage(be32I[2],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be15
        distX=556;
        distY=156;
        if(lineActive&&bits[15]&&frameActive) 
          offGraphics.drawImage(be32I[3],distX,distY,this);
        else if(!lineActive&&bits[15]) 
          offGraphics.drawImage(be32I[1],distX,distY,this);
        else if(lineActive&&!bits[15]&&frameActive) 
          offGraphics.drawImage(be32I[2],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // putting second frame (be13-be15) on sheet
        distX=455;
        distY=126;
        if(lineActive&&!frameActive) {
          offGraphics.drawImage(frame3I[2],distX,distY,this);
        }
        else if(lineActive&&frameActive) {
          offGraphics.drawImage(frame3I[3],distX,distY,this);
        }
        else offGraphics.drawImage(frame3I[0],distX,distY,this);
      }
      // putting be switches in first frame
      if(power[4]) {
        // be5
        distX=207;
        distY=131;
        if(bits[5]) {offGraphics.drawImage(be32I[3],distX,distY,this);frameActive=true;}
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be6
        distX=241;
        distY=143;
        if(bits[6]) {offGraphics.drawImage(be32I[3],distX,distY,this);frameActive=true;}
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be7
        distX=276;
        distY=156;
        if(bits[7]) {offGraphics.drawImage(be32I[3],distX,distY,this);frameActive=true;}
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // putting first frame (be5-be7) on sheet
        distX=175;
        distY=126;
        if((bits[4]&&power[4])||frameActive) { 
          offGraphics.drawImage(frame3I[3],distX,distY,this);
        }
        else offGraphics.drawImage(frame3I[1],distX,distY,this);
      }
      else {
      // be5
        distX=207;
        distY=131;
        if(bits[5]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be6
        distX=241;
        distY=143;
        if(bits[6]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be7
        distX=276;
        distY=156;
        if(bits[7]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // putting first frame (be5-be7) on sheet
        distX=175;
        distY=126;
        if(lineActive) offGraphics.drawImage(frame3I[3],distX,distY,this); 
        else offGraphics.drawImage(frame3I[0],distX,distY,this);
      }
    }  
    else {
      // putting first frame (be5-be7) on sheet
      distX=175;
      distY=126;
      offGraphics.drawImage(frame3I[1],distX,distY,this);
      // putting be switches in first frame
      // be5
      distX=207;
      distY=131;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be6
      distX=241;
      distY=143;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be7
      distX=276;
      distY=156;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // putting second frame (be13-be15) on sheet
      distX=455;
      distY=126;
      offGraphics.drawImage(frame3I[1],distX,distY,this);
      // putting be switches in second frame
      // be13
      distX=487;
      distY=131;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be14
      distX=521;
      distY=143;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be15
      distX=556;
      distY=156;
      offGraphics.drawImage(be32I[2],distX,distY,this);
    }
    // putting line on sheet
    distX=187;
    distY=126;
    if(on) {
      if(lineActive) offGraphics.drawImage(line3I[1],distX,distY,this);
      else offGraphics.drawImage(line3I[0],distX,distY,this);
    }
    else offGraphics.drawImage(line3I[0],distX,distY,this);
    // putting be switches on sheet
    distX=117;
    distY=118;
    int sum=35;
    for(int i=3; i<nr-4; i++) {
      if(on) {
        if(bits[i]) {
          if(power[i]) offGraphics.drawImage(beI[3],distX,distY,this);
          else offGraphics.drawImage(beI[1],distX,distY,this);
        }
        else {
          if(power[i]) offGraphics.drawImage(beI[2],distX,distY,this);
          else offGraphics.drawImage(beI[0],distX,distY,this);
        }
      }
      else offGraphics.drawImage(beI[2],distX,distY,this);
      distX+=sum;
    }
  }
  /** Method that puts the fourth level switches on the sheet. */
  public void putFiC() {
    // Putting front switch be(0) on sheet.
    int distX=159;
    int distY=122;
    if(on) {
      if(!bits[0]&&!bits[1]&&!bits[2]&&!bits[3])
        if(bits[4]) offGraphics.drawImage(front4I[3],distX,distY,this);
        else offGraphics.drawImage(front4I[2],distX,distY,this);
      else 
        if(bits[4]) offGraphics.drawImage(front4I[1],distX,distY,this);
        else offGraphics.drawImage(front4I[0],distX,distY,this);
    }
    else offGraphics.drawImage(front4I[2],distX,distY,this);
    if(on) {
      if(power[8]) {
        // be9
        distX=347;
        distY=199;
        if(bits[9]) offGraphics.drawImage(be32I[3],distX,distY,this);
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be10
        distX=382;
        distY=211;
        if(bits[10]) offGraphics.drawImage(be32I[3],distX,distY,this);
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be11
        distX=417;
        distY=224;
        if(bits[11]) offGraphics.drawImage(be32I[3],distX,distY,this);
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be12
        distX=451;
        distY=237;
        if(bits[12]) offGraphics.drawImage(be32I[3],distX,distY,this);
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be13
        distX=485;
        distY=249;
        if(bits[13]) offGraphics.drawImage(be32I[3],distX,distY,this);
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be14
        distX=519;
        distY=263;
        if(bits[14]) offGraphics.drawImage(be32I[3],distX,distY,this);
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // be15
        distX=554;
        distY=276;
        if(bits[15]) offGraphics.drawImage(be32I[3],distX,distY,this);
        else offGraphics.drawImage(be32I[2],distX,distY,this);
        // Putting frame on sheet
        distX=315;
        distY=194;
        if(bits[9]||bits[10]||bits[11]||bits[12]||bits[13]||bits[14]||bits[15]) 
          offGraphics.drawImage(frame4I[3],distX,distY,this);
        else if(bits[8]) 
          offGraphics.drawImage(frame4I[3],distX,distY,this);
        else 
          offGraphics.drawImage(frame4I[1],distX,distY,this);
      }
      else {
        // be9
        distX=347;
        distY=199;
        if(bits[9]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be10
        distX=382;
        distY=211;
        if(bits[10]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be11
        distX=417;
        distY=224;
        if(bits[11]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be12
        distX=451;
        distY=237;
        if(bits[12]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be13
        distX=485;
        distY=249;
        if(bits[13]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be14
        distX=519;
        distY=263;
        if(bits[14]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // be15
        distX=554;
        distY=276;
        if(bits[15]) offGraphics.drawImage(be32I[1],distX,distY,this);
        else offGraphics.drawImage(be32I[0],distX,distY,this);
        // Putting frame on sheet
        distX=315;
        distY=194;
        offGraphics.drawImage(frame4I[0],distX,distY,this);
      }
    }
    else {
      // Putting frame on sheet
      distX=315;
      distY=194;
      offGraphics.drawImage(frame4I[1],distX,distY,this);
      // be9
      distX=347;
      distY=199;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be10
      distX=382;
      distY=211;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be11
      distX=417;
      distY=224;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be12
      distX=451;
      distY=237;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be13
      distX=485;
      distY=249;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be14
      distX=519;
      distY=263;
      offGraphics.drawImage(be32I[2],distX,distY,this);
      // be15
      distX=554;
      distY=276;
      offGraphics.drawImage(be32I[2],distX,distY,this);
    }
    
    // Putting be switches on sheet
    distX=187;
    distY=186;
    int sum=35;
    for(int i=5; i<nr-8; i++) {
      if(on) {
        if(bits[i]) {
          if(power[i]) offGraphics.drawImage(beI[3],distX,distY,this);
          else offGraphics.drawImage(beI[1],distX,distY,this);
        }
        else {
          if(power[i]) offGraphics.drawImage(beI[2],distX,distY,this);
          else offGraphics.drawImage(beI[0],distX,distY,this);
        }
      }
      else offGraphics.drawImage(beI[2],distX,distY,this);
      distX+=sum;
    }
  }
  /**
   * Puts b82, Fm, Fl, Fk and Fi on sheet.
   */
  public void putDenotationC() {
    // Setting color
    offGraphics.setColor(Color.black);
    offGraphics.drawString("b82",638,40);
    offGraphics.drawString("Fm",638,61);
    offGraphics.drawString("Fl",638,111);
    offGraphics.drawString("Fk",638,177);
    offGraphics.drawString("Fi",638,298);
  }
  /**
   * This method realizes an increasing progross bar.
   * @param g graphics variable in which the bar will be placed.
   * @param images number of images.
   */
  public void buildProgress(Graphics g,int images) {
    int factor=30;
    // scaling
    Dimension d=size();
    // Setting color
    g.setColor(Color.blue);
    int x1=d.width/2-(images*factor)/2;
    // drawing a rectangle round the bar
    g.drawRect(x1,d.height/2+20,factor*images,20);
    // realizing the increasing bar
    for(int i=0; i<=images; i++) {
      g.fillRect(x1,d.height/2+20,i*factor,20);
      try {
        tracker.waitForID(i);
      } catch (InterruptedException ie) {}
    }
    g.setColor(Color.black);
  }
  public void run() {
    try {
      // Start downloading the images. Wait until they're loaded
      // before requesting repaints.
      tracker.waitForAll();
    } catch (InterruptedException ie) {}
    while(Thread.currentThread()==thread) {
      // Working with reduced performance
      Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
      repaint();
      try {
        thread.sleep(500);
      }catch (InterruptedException ie) {}
    }
  }
  public void paint(Graphics g) {
    update(g);
  }
  public void update(Graphics g) {
    // scaling
    Dimension d=size();
    // wait until all images are loaded
    if (!tracker.checkAll()) {
      g.setColor(Color.white);
      g.fillRect(0, 0, d.width, d.height);
      g.setColor(Color.black);
      g.drawString("Please wait, loading...",(d.width/2-80),d.height/2);
      buildProgress(g,13);
    }
    // now start drawing !!!
    else {
		  if ( (offGraphics==null) 
		  		|| (d.width!=offDimension.width) 
			  	|| (d.height!=offDimension.height) ) {
        offDimension=d;
        offImage=createImage(d.width, d.height);
        offGraphics=offImage.getGraphics();
      }
      // drawing white sheet
      offGraphics.setColor(Color.white);
      offGraphics.fillRect(0,0,d.width,d.height);
      offGraphics.setColor(Color.black);
      // drawing control buttons
      putControlsC();
      // placing bit buttons
      putBitButtonsC();
      // placing first level on sheet
      putFmC();
      // placing second level on sheet
      putFlC();
      // placing third level on sheet
      putFkC(); 
      // placing fourth level on sheet
      putFiC();
      // drawing denotation
      putDenotationC();
      // painting Buffer on Applet
      g.drawImage(offImage,0,0,this);
    }
  }
}
