import java.applet.Applet;
import java.awt.*;
/**
 * Applet which simulates the Z3 (1941 Konrad Zuse) shifter.
 * It simulates a 16-bit integer shifter.
 * @author Cneyt Gktekin
 * @version 1.5
 */
public class Shifter extends Applet implements Runnable {
  // needed for -double buffering-
  Dimension offDimension;
  Image offImage;
  Graphics offGraphics;
  /** number of bits. */
  int nr=16;
  /** Some booleans to Control the circuit. */
  /** ON / OFF buttons. */
  boolean on=false;
  /** Reset button. */
  boolean c=false;
  /** fhON: 16bit right shift. */
  boolean fhON=false;
  /** fiON: 8bit left shift. */
  boolean fiON=false;
  /** fkON: 4bit left shift. */
  boolean fkON=false;
  /** flON: 2bit left shift. */
  boolean flON=false;
  /** fmON: 1bit left shift. */
  boolean fmON=false;
  /** Input bits (Fh) stored in a boolean array. */
  boolean[] bits;
  /** Bits for the Fi phase 32bits long. */
  boolean[] bitsFi;
  /** Bits for the Fk phase 24bits long. */
  boolean[] bitsFk;
  /** Bits for the Fl phase 20bits long. */
  boolean[] bitsFl;
  /** Bits for the Fm phase 2bits long. */
  boolean[] bitsFm;
  // Image handling
  /** Control images. */
  Image resetONI;
  Image resetOFFI;
  Image onDI;
  Image onAI;
  Image offI;
  Image fhONI;
  Image fhOFFI;
  Image fiONI;
  Image fiOFFI;
  Image fkONI;
  Image fkOFFI;
  Image flONI;
  Image flOFFI;
  Image fmONI;
  Image fmOFFI;
  /** small ground image. */
  Image[] endI;
  /** small ground image. */
  Image[] endRI;
  /** bits. */
  Image[] bitsI;
  /** Images for the 16bit (Fh) right shifter. */
  Image[] linesFhI;
  /** Images for the 8bit (Fi) left shifter. */
  Image[] linesFiI;
  /** Images for the 4bit (Fk) left shifter. */
  Image[] linesFkI;
  /** Images for the 2bit (Fl) left shifter. */
  Image[] linesFlI;
  /** Images for the 1bit (Fm) left shifter. */
  Image[] linesFmI;
  /** Media tracker to keep loading under control. */
  MediaTracker tracker;
  /** The main simulation thread. */
  private Thread thread=null;
  /** 
   * Method realizes the reset button (C).
   * It creates a new array with false values.
   */
  private void c() {
    // Initializing bits with 0 (false)
    fhON=false;fiON=false;fkON=false;flON=false;fmON=false;
    bits=new boolean[nr];
    bitsFi=new boolean[nr*2];
    bitsFk=new boolean[nr+8];
    bitsFl=new boolean[nr+4];
    bitsFm=new boolean[nr+2];
    start();
  }
  /**
   * Loading Images and some other initialization.
   */
  public void init() {
    // Initializing bits
    c();
    // Loading images
    // first create a new MediaTracker object
    tracker=new MediaTracker(this);
    // control images
    onAI=getImage(getCodeBase(),"Images/onA.gif");
    tracker.addImage(onAI,0);
    onDI=getImage(getCodeBase(),"Images/onD.gif");
    tracker.addImage(onDI,0);
    offI=getImage(getCodeBase(),"Images/off.gif");
    tracker.addImage(offI,0);
    resetOFFI=getImage(getCodeBase(),"Images/resetOff.gif");
    tracker.addImage(resetOFFI,0);
    resetONI=getImage(getCodeBase(),"Images/resetOn.gif");
    tracker.addImage(resetONI,0);
    fhONI=getImage(getCodeBase(),"Images/fhON.GIF");
    tracker.addImage(fhONI,0);
    fhOFFI=getImage(getCodeBase(),"Images/fhOFF.GIF");
    tracker.addImage(fhOFFI,0);
    fiONI=getImage(getCodeBase(),"Images/fiON.GIF");
    tracker.addImage(fiONI,0);
    fiOFFI=getImage(getCodeBase(),"Images/fiOFF.GIF");
    tracker.addImage(fiOFFI,0);
    fkONI=getImage(getCodeBase(),"Images/fkON.GIF");
    tracker.addImage(fkONI,0);
    fkOFFI=getImage(getCodeBase(),"Images/fkOFF.GIF");
    tracker.addImage(fkOFFI,0);
    flONI=getImage(getCodeBase(),"Images/flON.GIF");
    tracker.addImage(flONI,0);
    flOFFI=getImage(getCodeBase(),"Images/flOFF.GIF");
    tracker.addImage(flOFFI,0);
    fmONI=getImage(getCodeBase(),"Images/fmON.GIF");
    tracker.addImage(fhONI,0);
    fmOFFI=getImage(getCodeBase(),"Images/fmOFF.GIF");
    tracker.addImage(fmOFFI,0);
    // small ground images
    endI=new Image[2];
    for(int i=0; i<2; i++) {
      endI[i]=getImage(getCodeBase(),"Images/Fh/end"+i+".gif");
      tracker.addImage(endI[i],1);
    }
    // small reflected ground images
    endRI=new Image[2];
    for(int i=0; i<2; i++) {
      endRI[i]=getImage(getCodeBase(),"Images/Fi/end"+i+".gif");
      tracker.addImage(endRI[i],2);
    }
    // bit images
    bitsI=new Image[4];
    for(int i=0; i<4; i++) {
      bitsI[i]=getImage(getCodeBase(),"Images/bit"+i+".gif");
      tracker.addImage(bitsI[i],3);
    }
    // Fh Images
    linesFhI=new Image[8];
    for(int i=0; i<8; i++) {
      linesFhI[i]=getImage(getCodeBase(),"Images/Fh/line"+i+".gif");
      tracker.addImage(linesFhI[i],4);
    }
    // Fi Images
    linesFiI=new Image[6];
    for(int i=0; i<6; i++) {
      linesFiI[i]=getImage(getCodeBase(),"Images/Fi/line"+i+".gif");
      tracker.addImage(linesFiI[i],5);
    }
    // Fk Images
    linesFkI=new Image[2];
    for(int i=0; i<2; i++) {
      linesFkI[i]=getImage(getCodeBase(),"Images/Fk/line"+i+".gif");
      tracker.addImage(linesFkI[i],6);
    }
    // Fl Images
    linesFlI=new Image[2];
    for(int i=0; i<2; i++) {
      linesFlI[i]=getImage(getCodeBase(),"Images/Fl/line"+i+".gif");
      tracker.addImage(linesFlI[i],7);
    }
    // Fm Images
    linesFmI=new Image[2];
    for(int i=0; i<2; i++) {
      linesFmI[i]=getImage(getCodeBase(),"Images/Fm/line"+i+".gif");
      tracker.addImage(linesFmI[i],8);
    }
  }
  public void start() {
    if (thread==null) {
      thread=new Thread(this);
      thread.start();
    }
    else {
      thread=null;
      thread=new Thread(this);
      thread.start();
    }
  }
  public void stop() {
    thread=null;
  }
  /** 
   * Standard mouse event Method.
   * In this Method the clickable areas are defined.
   * @param e MouseEvent
   * @param x x coordinate 
   * @param y y coordinate
   */
  public boolean mouseDown(Event e,int x,int y) {
    // Managing behaviour on ON button
    int x1Local=400;
    int y1Local=30;
    if((x>x1Local)&&(x<(x1Local+30))&&(y<y1Local+30)&&(y>y1Local)) {
      on=true;start();
    }
    // Managing behaviour on OFF button
    x1Local=370;
    y1Local=30;
    if((x>x1Local)&&(x<(x1Local+30))&&(y<y1Local+30)&&(y>y1Local)) {
      c();on=false;start();
    }
    // Managing behaviour on reset button
    x1Local=450;
    y1Local=30;
    if((x>x1Local)&&(x<(x1Local+40))&&(y<y1Local+40)&&(y>y1Local)) {
      c=(c ? false:true);start();
    }
    // Managing behaviour on Fh button
    x1Local=4;
    y1Local=45;
    if((x>x1Local)&&(x<(x1Local+25))&&(y<y1Local+25)&&(y>y1Local)) {
      fhON=(fhON ? false:true);start();
    }
    // Managing behaviour on Fi button
    x1Local=4;
    y1Local=140;
    if((x>x1Local)&&(x<(x1Local+25))&&(y<y1Local+25)&&(y>y1Local)) {
      fiON=(fiON ? false:true);start();
    }
    // Managing behaviour on Fk button
    x1Local=4;
    y1Local=240;
    if((x>x1Local)&&(x<(x1Local+25))&&(y<y1Local+25)&&(y>y1Local)) {
      fkON=(fkON ? false:true);start();
    }
    // Managing behaviour on Fl button
    x1Local=4;
    y1Local=340;
    if((x>x1Local)&&(x<(x1Local+25))&&(y<y1Local+25)&&(y>y1Local)) {
      flON=(flON ? false:true);start();
    }
    // Managing behaviour on Fm button
    x1Local=4;
    y1Local=440;
    if((x>x1Local)&&(x<(x1Local+25))&&(y<y1Local+25)&&(y>y1Local)) {
      fmON=(fmON ? false:true);start();
    }
    // Managing behaviour on bits
    x1Local=40;
    for(int i=0; i<nr; i++) {
      if((x>x1Local)&&(x<(x1Local+13))&&(y<33)&&(y>20)) {
        bits[i]=(bits[i] ? false:true);start();
      }
      x1Local+=15+((i%2==0) ? 2:0);
    }
    return true;
  }
  /** Method that puts the control buttons on the sheet. */
  private void putControlsC() {
    // putting ON/OFF button on sheet
    int distX=370;
    int distY=30;
    if(on) {
      offGraphics.drawImage(offI,distX,distY,this);
      offGraphics.drawImage(onAI,distX+30,distY,this);
    }
    else {
      offGraphics.drawImage(offI,distX,distY,this);
      offGraphics.drawImage(onDI,distX+30,distY,this);
    }
    // putting reset button on sheet
    distX=450;
    distY=30;
    offGraphics.drawImage(resetOFFI,distX,distY,this);
    if(c) {
      offGraphics.drawImage(resetOFFI,distX,distY,this);
      c();
      offGraphics.drawImage(resetONI,distX,distY,this);
      c=false;
      on=false;
    }
  }  
  /** Method that puts the shift buttons on the sheet. */
  private void putButtonsC() {
    // putting Fh button
    int distX=4;
    int distY=45;
    if(fhON) offGraphics.drawImage(fhONI,distX,distY,this);
    else offGraphics.drawImage(fhOFFI,distX,distY,this);
    // putting Fi button
    distX=4;
    distY=140;
    if(fiON) offGraphics.drawImage(fiONI,distX,distY,this);
    else offGraphics.drawImage(fiOFFI,distX,distY,this);
    // putting Fk button
    distX=4;
    distY=240;
    if(fkON) offGraphics.drawImage(fkONI,distX,distY,this);
    else offGraphics.drawImage(fkOFFI,distX,distY,this);
    // putting Fl button
    distX=4;
    distY=340;
    if(flON) offGraphics.drawImage(flONI,distX,distY,this);
    else offGraphics.drawImage(flOFFI,distX,distY,this);
    // putting Fm button
    distX=4;
    distY=440;
    if(fmON) offGraphics.drawImage(fmONI,distX,distY,this);
    else offGraphics.drawImage(fmOFFI,distX,distY,this);
  }
  /** 
   * Method that puts the bits on the sheet.
   */  
  private void putBitsC() {
    int distX=40;
    int distY=20;
    int sum=16;
    // putting bits
    for(int i=0; i<nr; i++) {
      if(fhON) {
        if(bits[i]) offGraphics.drawImage(bitsI[3],distX,distY,this);
        else offGraphics.drawImage(bitsI[1],distX,distY,this);
      }
      else {
        if(bits[i]) offGraphics.drawImage(bitsI[2],distX,distY,this);
        else offGraphics.drawImage(bitsI[0],distX,distY,this);
      }
      distX+=sum;
    }
  }
  /** 
   * Method that puts all lines for the 16-bit right shifting part
   * (Fh) of the Circuit, and realizes the ON/OFF Button.
   */
  private void putFhPartC() {
    // putting vertical lines on sheet
    int distX=36;
    int distY=60;
    int sum=16;
    for(int i=0; i<nr; i++) {
      if(fhON&&on) {
        bitsFi[i]=false;
        if(fiON&&on) offGraphics.drawImage(linesFhI[1],distX,distY,this);
        else offGraphics.drawImage(linesFhI[0],distX,distY,this);
      }
      else {
        if(bits[i]&&on) {
          bitsFi[i]=true;
          if(fiON) offGraphics.drawImage(linesFhI[3],distX,distY,this);
          else offGraphics.drawImage(linesFhI[2],distX,distY,this);
        }
        else {
          bitsFi[i]=false;
          if(fiON) offGraphics.drawImage(linesFhI[1],distX,distY,this);
          else offGraphics.drawImage(linesFhI[0],distX,distY,this);
        }
      }
      distX+=sum;
      
    }
    // putting diagonal lines and the small ground on sheet
    distX=49;
    distY=60;
    // variable to run through bitsFi
    int j=16;
    for(int i=0; i<nr-1; i++) {
      if(bits[i]&&fhON&&on) {
        bitsFi[j]=true;
        if(fiON) offGraphics.drawImage(linesFhI[7],distX,distY,this);
        else offGraphics.drawImage(linesFhI[6],distX,distY,this);
      }
      else {
        bitsFi[j]=false;
        if(fiON) offGraphics.drawImage(linesFhI[5],distX,distY,this);
        else offGraphics.drawImage(linesFhI[4],distX,distY,this);
      }
      j++;
      distX+=sum;
    }
    // putting last ground on the sheet
    if(bits[nr-1]&&fhON&&on) offGraphics.drawImage(endI[1],distX,distY,this);
    else offGraphics.drawImage(endI[0],distX,distY,this);
  }
  /** 
   * Method that puts all lines for the 8-bit left shifting part
   * (Fi) of the Circuit.
   */
  private void putFiPartC() {
    // putting vertical lines on sheet
    int distX=43;
    int distY=158;
    int sum=16;
    int j=8;
    for(int i=0; i<nr+7; i++) {
      if((bitsFi[i]&&!fiON)||(bitsFi[j]&&fiON)) {
        bitsFk[i]=true;
        if(fkON) offGraphics.drawImage(linesFiI[3],distX,distY,this);
        else offGraphics.drawImage(linesFiI[2],distX,distY,this);
      }
      else {
        bitsFk[i]=false;
        if(fkON) offGraphics.drawImage(linesFiI[1],distX,distY,this);
        else offGraphics.drawImage(linesFiI[0],distX,distY,this);
      }
      j++;
      distX+=sum;
    }
    // putting front grounds on sheet
    distX=34;
    distY=158;
    sum=16;
    for(int i=0; i<nr-8; i++) {
      if(bitsFi[i]&&fiON) {
        offGraphics.drawImage(endRI[1],distX,distY,this);
      }
      else {
        offGraphics.drawImage(endRI[0],distX,distY,this);
      }
      distX+=sum;
    }
    // putting rear grounds on sheet
    distX=410;
    distY=158;
    sum=16;
    for(int i=23; i<nr+15; i++) {
      if(bitsFi[i]&&!fiON) {
        offGraphics.drawImage(endI[1],distX,distY,this);
      }
      else {
        offGraphics.drawImage(endI[0],distX,distY,this);
      }
      distX+=sum;
    }
    // putting diagonal lines on sheet
    distX=44;
    distY=158;
    sum=16;
    j=0;
    for(int i=8; i<nr+15; i++) {
      if((bitsFi[i]&&fiON)||(bitsFi[j]&&!fiON)) {
        bitsFk[j]=true;
        offGraphics.drawImage(linesFiI[5],distX,distY,this);
      }
      else {
        bitsFk[j]=false;
        offGraphics.drawImage(linesFiI[4],distX,distY,this);
      }
      j++;
      distX+=sum;
    }
  } 
  /** 
   * Method that puts all lines for the 4-bit shifting part
   * (Fk) of the Circuit.
   */
  private void putFkPartC() {
    // putting vertical lines on sheet
    int distX=50;
    int distY=258;
    int sum=16;
    int j=4;
    for(int i=0; i<nr+3; i++) {
      if((bitsFk[i]&&!fkON)||(bitsFk[j]&&fkON)) {
        bitsFl[i]=true;
        if(flON) offGraphics.drawImage(linesFiI[3],distX,distY,this);
        else offGraphics.drawImage(linesFiI[2],distX,distY,this);
      }
      else {
        bitsFl[i]=false;
        if(flON) offGraphics.drawImage(linesFiI[1],distX,distY,this);
        else offGraphics.drawImage(linesFiI[0],distX,distY,this);
      }
      j++;
      distX+=sum;
    }
    // putting front grounds on sheet
    distX=41;
    distY=258;
    sum=16;
    for(int i=0; i<nr-12; i++) {
      if(bitsFk[i]&&fkON) {
        offGraphics.drawImage(endRI[1],distX,distY,this);
      }
      else {
        offGraphics.drawImage(endRI[0],distX,distY,this);
      }
      distX+=sum;
    }
    // putting rear gounds on sheet
    distX=354;
    distY=258;
    sum=16;
    for(int i=19; i<nr+7; i++) {
      if(bitsFk[i]&&!fkON) {
        offGraphics.drawImage(endI[1],distX,distY,this);
      }
      else {
        offGraphics.drawImage(endI[0],distX,distY,this);
      }
      distX+=sum;
    }
    // putting diagonal lines on sheet
    distX=50;
    distY=258;
    sum=16;
    j=0;
    for(int i=4; i<nr+7; i++) {
      if((bitsFk[i]&&fkON)||(bitsFk[j]&&!fkON)) {
        bitsFl[j]=true;
        offGraphics.drawImage(linesFkI[1],distX,distY,this);
      }
      else {
        bitsFl[j]=false;
        offGraphics.drawImage(linesFkI[0],distX,distY,this);
      }
      j++;
      distX+=sum;
    }
  }
  /** 
   * Method that puts all lines for the 2-bit shifting part
   * (Fl) of the Circuit.
   */
  private void putFlPartC() {
    // putting vertical lines on sheet
    int distX=57;
    int distY=358;
    int sum=16;
    int j=2;
    for(int i=0; i<nr+1; i++) {
      if((bitsFl[i]&&!flON)||(bitsFl[j]&&flON)) {
        bitsFm[i]=true;
        if(fmON) offGraphics.drawImage(linesFiI[3],distX,distY,this);
        else offGraphics.drawImage(linesFiI[2],distX,distY,this);
      }
      else {
        bitsFm[i]=false;
        if(fmON) offGraphics.drawImage(linesFiI[1],distX,distY,this);
        else offGraphics.drawImage(linesFiI[0],distX,distY,this);
      }
      j++;
      distX+=sum;
    }
    // putting front grounds on sheet
    distX=48;
    distY=358;
    sum=16;
    for(int i=0; i<nr-14; i++) {
      if(bitsFl[i]&&flON) {
        offGraphics.drawImage(endRI[1],distX,distY,this);
      }
      else {
        offGraphics.drawImage(endRI[0],distX,distY,this);
      }
      distX+=sum;
    }
    
    // putting rear gounds on sheet
    distX=329;
    distY=358;
    sum=16;
    for(int i=17; i<nr+3; i++) {
      if(bitsFl[i]&&!flON) {
        offGraphics.drawImage(endI[1],distX,distY,this);
      }
      else {
        offGraphics.drawImage(endI[0],distX,distY,this);
      }
      distX+=sum;
    }
    // putting diagonal lines on sheet
    distX=57;
    distY=358;
    sum=16;
    j=0;
    for(int i=2; i<nr+3; i++) {
      if((bitsFl[i]&&flON)||(bitsFl[j]&&!flON)) {
        bitsFm[j]=true;
        offGraphics.drawImage(linesFlI[1],distX,distY,this);
      }
      else {
        bitsFm[j]=false;
        offGraphics.drawImage(linesFlI[0],distX,distY,this);
      }
      j++;
      distX+=sum;
    }
  }
  /** 
   * Method that puts all lines for the 1-bit shifting part
   * (Fm) of the Circuit.
   */
  private void putFmPartC() {
    // putting result lines on sheet
    int distX=62;
    int distY=458;
    int sum=16;
    if(!fmON) {
      for(int i=0; i<nr; i++) {
        if(bitsFm[i]) {
          offGraphics.drawImage(linesFmI[1],distX,distY,this);
        }
        else {
          offGraphics.drawImage(linesFmI[0],distX,distY,this);
        }
        distX+=sum;
      }
    }
    else {
      for(int i=1; i<nr+1; i++) {
        if(bitsFm[i]) {
          offGraphics.drawImage(linesFmI[1],distX,distY,this);
        }
        else {
          offGraphics.drawImage(linesFmI[0],distX,distY,this);
        }
        distX+=sum;
      }
    }
  }
  /**
   * This method realizes an increasing progross bar.
   * @param g graphics variable in which the bar will be placed.
   * @param images number of images.
   */
  public void buildProgress(Graphics g,int images) {
    int factor=30;
    // scaling
    Dimension d=size();
    // Setting color
    g.setColor(Color.blue);
    int x1=d.width/2-(images*factor)/2;
    // drawing a rectangle round the bar
    g.drawRect(x1,d.height/2+20,factor*images,20);
    // realizing the increasing bar
    for(int i=0; i<=images; i++) {
      g.fillRect(x1,d.height/2+20,i*factor,20);
      try {
        tracker.waitForID(i);
      } catch (InterruptedException ie) {}
    }
    g.setColor(Color.black);
  }
  public void run() {
    try {
      // Start downloading the images. Wait until they're loaded
      // before requesting repaints.
      tracker.waitForAll();
    } catch (InterruptedException ie) {}
    while(Thread.currentThread()==thread) {
      // Working with reduced performance
      Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
      repaint();
      try {
        thread.sleep(500);
      }catch (InterruptedException ie) {}
    }
  }
  public void paint(Graphics g) {
    update(g);
  }
  public void update(Graphics g) {
    // scaling
    Dimension d=size();
    // wait until all images are loaded
    if (!tracker.checkAll()) {
      g.setColor(Color.white);
      g.fillRect(0, 0, d.width, d.height);
      g.setColor(Color.black);
      g.drawString("Please wait, loading...",(d.width/2-80),d.height/2);
      buildProgress(g,8);
    }
    // now start drawing !!!
    else {
		  if ( (offGraphics==null) 
		  		|| (d.width!=offDimension.width) 
			  	|| (d.height!=offDimension.height) ) {
        offDimension=d;
        offImage=createImage(d.width, d.height);
        offGraphics=offImage.getGraphics();
      }
      // drawing white sheet
      offGraphics.setColor(Color.white);
      offGraphics.fillRect(0,0,d.width,d.height);
      offGraphics.setColor(Color.black);
      // drawing control buttons
      putControlsC();
      // drawing shift buttons
      putButtonsC();
      // drawing bits
      putBitsC();
      // drawing Fh part
      putFhPartC();
      // drawing Fi part
      putFiPartC();
      // drawing Fk part
      putFkPartC();
      // drawing Fl part
      putFlPartC();
      // drawing Fm part
      putFmPartC();
      // painting Buffer on Applet
      g.drawImage(offImage,0,0,this);
    }
  }
}
